/*
 * Copyright 2018 Sony Corporation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>

/*
 * sync with firmup_priv.h
 */
struct firmup_header {
	uint32_t sz;
	uint8_t reserved[0x10 - sizeof (uint32_t)];
} __attribute__((packed));


static int do_loadfirm(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	const struct firmup_header *p_header;
	const void *src;
	void *dst;
	uint32_t size_max;
	uint32_t size;

	if (argc < 4) {
		puts("Too few arguments.\n");
		return 1;
	}

	p_header = (const struct firmup_header *)simple_strtoul(argv[1], NULL, 16);
	dst = (void *)simple_strtoul(argv[2], NULL, 16);
	size_max = simple_strtoul(argv[3], NULL, 16);

	size = p_header->sz;
	src = ((const void *)p_header) + sizeof(*p_header);

	if (size > size_max) {
		printf("size(%x) is too large.\n", size);
		return 1;
	}

	memcpy(dst, src, size);

	printf("%u byte copied.\n", size);

	return 0;
}

U_BOOT_CMD(
	/* name = */loadfirm,
	/* maxargs = */ 4,
	/* repeatable = */ 0,
	/* command = */do_loadfirm,
	/* usage = */ "load firmware with nasca format",
	/* help = */ "src dst max_size"
);
